"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import Optional
from typing_extensions import Annotated
from access_server_connector_v2.models.deactivate_in import DeactivateIn
from access_server_connector_v2.models.grant_in import GrantIn
from access_server_connector_v2.models.wrapped_response_id_out import WrappedResponseIdOut
from access_server_connector_v2.models.wrapped_response_int import WrappedResponseInt
from access_server_connector_v2.models.wrapped_response_list_owned_package_out import WrappedResponseListOwnedPackageOut
from access_server_connector_v2.models.wrapped_response_list_ownership_reason_out import WrappedResponseListOwnershipReasonOut

from access_server_connector_v2.api_client import ApiClient, RequestSerialized
from access_server_connector_v2.api_response import ApiResponse
from access_server_connector_v2.rest import RESTResponseType


class OwnershipApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    async def deactivate_v1_ownership_reasons_reason_id_deactivate_post(
        self,
        reason_id: StrictInt,
        deactivate_in: DeactivateIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WrappedResponseIdOut:
        """Deactivate (revoke) an ownership reason


        :param reason_id: (required)
        :type reason_id: int
        :param deactivate_in: (required)
        :type deactivate_in: DeactivateIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_v1_ownership_reasons_reason_id_deactivate_post_serialize(
            reason_id=reason_id,
            deactivate_in=deactivate_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def deactivate_v1_ownership_reasons_reason_id_deactivate_post_with_http_info(
        self,
        reason_id: StrictInt,
        deactivate_in: DeactivateIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WrappedResponseIdOut]:
        """Deactivate (revoke) an ownership reason


        :param reason_id: (required)
        :type reason_id: int
        :param deactivate_in: (required)
        :type deactivate_in: DeactivateIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_v1_ownership_reasons_reason_id_deactivate_post_serialize(
            reason_id=reason_id,
            deactivate_in=deactivate_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def deactivate_v1_ownership_reasons_reason_id_deactivate_post_without_preload_content(
        self,
        reason_id: StrictInt,
        deactivate_in: DeactivateIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deactivate (revoke) an ownership reason


        :param reason_id: (required)
        :type reason_id: int
        :param deactivate_in: (required)
        :type deactivate_in: DeactivateIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_v1_ownership_reasons_reason_id_deactivate_post_serialize(
            reason_id=reason_id,
            deactivate_in=deactivate_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _deactivate_v1_ownership_reasons_reason_id_deactivate_post_serialize(
        self,
        reason_id,
        deactivate_in,
        origin,
        referer,
        portal_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if reason_id is not None:
            _path_params['reason_id'] = reason_id
        # process the query parameters
        # process the header parameters
        if origin is not None:
            _header_params['origin'] = origin
        if referer is not None:
            _header_params['referer'] = referer
        if portal_id is not None:
            _header_params['portal_id'] = portal_id
        # process the form parameters
        # process the body parameter
        if deactivate_in is not None:
            _body_params = deactivate_in


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Bearer Authorization', 
            'ServerName Authorization'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/ownership/reasons/{reason_id}/deactivate',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def grant_v1_ownership_grant_post(
        self,
        grant_in: GrantIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WrappedResponseIdOut:
        """Manually grant an access package


        :param grant_in: (required)
        :type grant_in: GrantIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_v1_ownership_grant_post_serialize(
            grant_in=grant_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def grant_v1_ownership_grant_post_with_http_info(
        self,
        grant_in: GrantIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WrappedResponseIdOut]:
        """Manually grant an access package


        :param grant_in: (required)
        :type grant_in: GrantIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_v1_ownership_grant_post_serialize(
            grant_in=grant_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def grant_v1_ownership_grant_post_without_preload_content(
        self,
        grant_in: GrantIn,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Manually grant an access package


        :param grant_in: (required)
        :type grant_in: GrantIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_v1_ownership_grant_post_serialize(
            grant_in=grant_in,
            origin=origin,
            referer=referer,
            portal_id=portal_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _grant_v1_ownership_grant_post_serialize(
        self,
        grant_in,
        origin,
        referer,
        portal_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        if origin is not None:
            _header_params['origin'] = origin
        if referer is not None:
            _header_params['referer'] = referer
        if portal_id is not None:
            _header_params['portal_id'] = portal_id
        # process the form parameters
        # process the body parameter
        if grant_in is not None:
            _body_params = grant_in


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Bearer Authorization', 
            'ServerName Authorization'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/ownership/grant',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def member_packages_count_v1_ownership_members_member_id_packages_count_get(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WrappedResponseInt:
        """Count the access packages a member owns

        Query: portal_id, active_only?. Total matching packages (for pagination).

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_count_v1_ownership_members_member_id_packages_count_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseInt",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def member_packages_count_v1_ownership_members_member_id_packages_count_get_with_http_info(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WrappedResponseInt]:
        """Count the access packages a member owns

        Query: portal_id, active_only?. Total matching packages (for pagination).

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_count_v1_ownership_members_member_id_packages_count_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseInt",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def member_packages_count_v1_ownership_members_member_id_packages_count_get_without_preload_content(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Count the access packages a member owns

        Query: portal_id, active_only?. Total matching packages (for pagination).

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_count_v1_ownership_members_member_id_packages_count_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseInt",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _member_packages_count_v1_ownership_members_member_id_packages_count_get_serialize(
        self,
        member_id,
        portal_id,
        active_only,
        origin,
        referer,
        portal_id2,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if member_id is not None:
            _path_params['member_id'] = member_id
        # process the query parameters
        if portal_id is not None:
            
            _query_params.append(('portal_id', portal_id))
            
        if active_only is not None:
            
            _query_params.append(('active_only', active_only))
            
        # process the header parameters
        if origin is not None:
            _header_params['origin'] = origin
        if referer is not None:
            _header_params['referer'] = referer
        if portal_id2 is not None:
            _header_params['portal_id'] = portal_id2
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer Authorization', 
            'ServerName Authorization'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/ownership/members/{member_id}/packages/count',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def member_packages_v1_ownership_members_member_id_packages_get(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        limit: Optional[Annotated[int, Field(le=200, strict=True, ge=1)]] = None,
        offset: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WrappedResponseListOwnedPackageOut:
        """List the access packages a member owns (paginated)

        Query: portal_id, active_only?, limit, offset. active_only=true returns only currently-owned packages. Use …/packages/count for the total.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_v1_ownership_members_member_id_packages_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            limit=limit,
            offset=offset,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnedPackageOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def member_packages_v1_ownership_members_member_id_packages_get_with_http_info(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        limit: Optional[Annotated[int, Field(le=200, strict=True, ge=1)]] = None,
        offset: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WrappedResponseListOwnedPackageOut]:
        """List the access packages a member owns (paginated)

        Query: portal_id, active_only?, limit, offset. active_only=true returns only currently-owned packages. Use …/packages/count for the total.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_v1_ownership_members_member_id_packages_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            limit=limit,
            offset=offset,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnedPackageOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def member_packages_v1_ownership_members_member_id_packages_get_without_preload_content(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        active_only: Optional[StrictBool] = None,
        limit: Optional[Annotated[int, Field(le=200, strict=True, ge=1)]] = None,
        offset: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List the access packages a member owns (paginated)

        Query: portal_id, active_only?, limit, offset. active_only=true returns only currently-owned packages. Use …/packages/count for the total.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._member_packages_v1_ownership_members_member_id_packages_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            active_only=active_only,
            limit=limit,
            offset=offset,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnedPackageOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _member_packages_v1_ownership_members_member_id_packages_get_serialize(
        self,
        member_id,
        portal_id,
        active_only,
        limit,
        offset,
        origin,
        referer,
        portal_id2,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if member_id is not None:
            _path_params['member_id'] = member_id
        # process the query parameters
        if portal_id is not None:
            
            _query_params.append(('portal_id', portal_id))
            
        if active_only is not None:
            
            _query_params.append(('active_only', active_only))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if offset is not None:
            
            _query_params.append(('offset', offset))
            
        # process the header parameters
        if origin is not None:
            _header_params['origin'] = origin
        if referer is not None:
            _header_params['referer'] = referer
        if portal_id2 is not None:
            _header_params['portal_id'] = portal_id2
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer Authorization', 
            'ServerName Authorization'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/ownership/members/{member_id}/packages',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def ownership_list_v1_ownership_members_member_id_reasons_get(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WrappedResponseListOwnershipReasonOut:
        """List a member's ownership reasons

        Query: portal_id.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._ownership_list_v1_ownership_members_member_id_reasons_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnershipReasonOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def ownership_list_v1_ownership_members_member_id_reasons_get_with_http_info(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WrappedResponseListOwnershipReasonOut]:
        """List a member's ownership reasons

        Query: portal_id.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._ownership_list_v1_ownership_members_member_id_reasons_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnershipReasonOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def ownership_list_v1_ownership_members_member_id_reasons_get_without_preload_content(
        self,
        member_id: StrictInt,
        portal_id: StrictInt,
        origin: Optional[StrictStr] = None,
        referer: Optional[StrictStr] = None,
        portal_id2: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List a member's ownership reasons

        Query: portal_id.

        :param member_id: (required)
        :type member_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._ownership_list_v1_ownership_members_member_id_reasons_get_serialize(
            member_id=member_id,
            portal_id=portal_id,
            origin=origin,
            referer=referer,
            portal_id2=portal_id2,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WrappedResponseListOwnershipReasonOut",
            '422': "HTTPValidationError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _ownership_list_v1_ownership_members_member_id_reasons_get_serialize(
        self,
        member_id,
        portal_id,
        origin,
        referer,
        portal_id2,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if member_id is not None:
            _path_params['member_id'] = member_id
        # process the query parameters
        if portal_id is not None:
            
            _query_params.append(('portal_id', portal_id))
            
        # process the header parameters
        if origin is not None:
            _header_params['origin'] = origin
        if referer is not None:
            _header_params['referer'] = referer
        if portal_id2 is not None:
            _header_params['portal_id'] = portal_id2
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer Authorization', 
            'ServerName Authorization'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/ownership/members/{member_id}/reasons',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from access_server_connector_v2.models.course_structure import CourseStructure

class TestCourseStructure(unittest.TestCase):
    """CourseStructure unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> CourseStructure:
        """Test CourseStructure
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `CourseStructure`
        """
        model = CourseStructure()
        if include_optional:
            return CourseStructure(
                course_id = 56,
                version = 's_0',
                modules = [
                    access_server_connector_v2.models.module_structure.ModuleStructure(
                        module_id = 56, 
                        is_published = True, 
                        default_unlock = null, 
                        lessons = [
                            access_server_connector_v2.models.lesson_structure.LessonStructure(
                                lesson_id = 56, 
                                is_published = True, 
                                default_unlock = null, )
                            ], )
                    ]
            )
        else:
            return CourseStructure(
                course_id = 56,
        )
        """

    def testCourseStructure(self):
        """Test CourseStructure"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from access_server_connector_v2.models.event_order import EventOrder
from access_server_connector_v2.models.event_product import EventProduct
from access_server_connector_v2.models.event_subscription import EventSubscription
from access_server_connector_v2.models.reason_kind import ReasonKind
from access_server_connector_v2.models.reason_status import ReasonStatus
from typing import Optional, Set
from typing_extensions import Self
from pydantic_core import to_jsonable_python

class AccessOrderEvent(BaseModel):
    """
    AccessOrderEvent
    """ # noqa: E501
    event_id: StrictStr
    occurred_at: StrictInt
    portal_id: StrictInt
    member_id: StrictInt
    reason_kind: Optional[ReasonKind] = None
    product: Optional[EventProduct] = None
    access_package_ids: Optional[List[StrictInt]] = None
    order: Optional[EventOrder] = None
    status: ReasonStatus
    status_reason: Optional[StrictStr] = None
    subscription: Optional[EventSubscription] = None
    __properties: ClassVar[List[str]] = ["event_id", "occurred_at", "portal_id", "member_id", "reason_kind", "product", "access_package_ids", "order", "status", "status_reason", "subscription"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_by_alias=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(to_jsonable_python(self.to_dict()))

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AccessOrderEvent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of product
        if self.product:
            _dict['product'] = self.product.to_dict()
        # override the default output from pydantic by calling `to_dict()` of order
        if self.order:
            _dict['order'] = self.order.to_dict()
        # override the default output from pydantic by calling `to_dict()` of subscription
        if self.subscription:
            _dict['subscription'] = self.subscription.to_dict()
        # set to None if product (nullable) is None
        # and model_fields_set contains the field
        if self.product is None and "product" in self.model_fields_set:
            _dict['product'] = None

        # set to None if access_package_ids (nullable) is None
        # and model_fields_set contains the field
        if self.access_package_ids is None and "access_package_ids" in self.model_fields_set:
            _dict['access_package_ids'] = None

        # set to None if order (nullable) is None
        # and model_fields_set contains the field
        if self.order is None and "order" in self.model_fields_set:
            _dict['order'] = None

        # set to None if status_reason (nullable) is None
        # and model_fields_set contains the field
        if self.status_reason is None and "status_reason" in self.model_fields_set:
            _dict['status_reason'] = None

        # set to None if subscription (nullable) is None
        # and model_fields_set contains the field
        if self.subscription is None and "subscription" in self.model_fields_set:
            _dict['subscription'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AccessOrderEvent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "event_id": obj.get("event_id"),
            "occurred_at": obj.get("occurred_at"),
            "portal_id": obj.get("portal_id"),
            "member_id": obj.get("member_id"),
            "reason_kind": obj.get("reason_kind"),
            "product": EventProduct.from_dict(obj["product"]) if obj.get("product") is not None else None,
            "access_package_ids": obj.get("access_package_ids"),
            "order": EventOrder.from_dict(obj["order"]) if obj.get("order") is not None else None,
            "status": obj.get("status"),
            "status_reason": obj.get("status_reason"),
            "subscription": EventSubscription.from_dict(obj["subscription"]) if obj.get("subscription") is not None else None
        })
        return _obj


